"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const MarketSummaryResponse_1 = require("./response/MarketSummaryResponse");
const CryptoJs = require("crypto-js");
const Https = require("https");
const SignalR = require("signalr-client");
const url_1 = require("url");
const MarketUpdate_1 = require("./response/MarketUpdate");
const BuyLimitResponse_1 = require("./response/BuyLimitResponse");
class BittrexApiClient {
    constructor(apiKey, apiSecret) {
        this.apiKey = apiKey;
        this.apiSecret = apiSecret;
    }
    getMarketSummary(marketSummaryRequest) {
        return this.makeRequest("public/getmarketsummary", ["market", marketSummaryRequest.getName()])
            .then((jsonResponse) => {
            return new MarketSummaryResponse_1.MarketSummaryResponse(jsonResponse);
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    buyLimit(buyLimitRequest) {
        return this.makeRequest("/market/buylimit", ["market", buyLimitRequest.getMarket()], ["quantity", buyLimitRequest.getQuantity().toString()], ["rate", buyLimitRequest.getPrice().toString()])
            .then((jsonResponse) => {
            return new BuyLimitResponse_1.BuyLimitResponse(jsonResponse);
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getMarketUpdatesStream(marketUpdatesStreamRequest, callback) {
        let websocketClient = new SignalR.client(BittrexApiClient.WEB_SOCKET_HOST, BittrexApiClient.WEB_SOCKET_HUB);
        let watchableMarkets = marketUpdatesStreamRequest.getWatchableMarkets();
        websocketClient.serviceHandlers.connected = () => {
            for (let watchableMarket of watchableMarkets) {
                websocketClient.call(BittrexApiClient.WEB_SOCKET_HUB, BittrexApiClient.WEB_SOCKET_HUB_SUBSCRIBE_DELTAS, watchableMarket);
            }
        };
        websocketClient.serviceHandlers.messageReceived = (message) => {
            if (message.type !== "utf8") {
                return;
            }
            message = JSON.parse(message.utf8Data);
            let updates = message.M;
            if (updates === undefined || updates.length === 0) {
                return;
            }
            let marketUpdates = [];
            for (let updateJson of message.M) {
                console.log(updateJson);
                if (updateJson.M !== "updateExchangeState") {
                    return;
                }
                for (let marketUpdateJson of updateJson.A) {
                    marketUpdates.push(new MarketUpdate_1.MarketUpdate(marketUpdateJson));
                }
            }
            callback(marketUpdates);
        };
    }
    makeRequest(operation, ...parameters) {
        let apiEndpointUrl = new url_1.URL(BittrexApiClient.API_BASE_URL_PATH + operation, BittrexApiClient.API_BASE_URL_ROOT);
        apiEndpointUrl.searchParams.append(BittrexApiClient.API_API_KEY_PARAMETER, this.apiKey);
        apiEndpointUrl.searchParams.append(BittrexApiClient.API_TIMESTAMP_PARAMETER, new Date().getTime().toString());
        for (let parameter of parameters) {
            apiEndpointUrl.searchParams.append(parameter[0], parameter[1]);
        }
        let apiSign = CryptoJs.HmacSHA512(apiEndpointUrl.toString(), this.apiSecret);
        return new Promise((fulfill, reject) => {
            let clientRequest = Https.request(apiEndpointUrl, (bittrexResponse) => {
                bittrexResponse.on("data", (bittrexData) => {
                    bittrexData = JSON.parse(bittrexData.toString());
                    if (bittrexData.success) {
                        fulfill(bittrexData);
                    }
                    else {
                        reject(bittrexData.message);
                    }
                });
            });
            clientRequest.setHeader("apisign", apiSign);
            clientRequest.end();
        });
    }
}
BittrexApiClient.WEB_SOCKET_HOST = "wss://socket.bittrex.com/signalr";
BittrexApiClient.WEB_SOCKET_HUB = "CoreHub";
BittrexApiClient.WEB_SOCKET_HUB_SUBSCRIBE_DELTAS = "SubscribeToExchangeDeltas";
BittrexApiClient.API_BASE_URL_PATH = "/api/v1.1/";
BittrexApiClient.API_BASE_URL_ROOT = "https://www.bittrex.com/";
BittrexApiClient.API_API_KEY_PARAMETER = "apikey";
BittrexApiClient.API_TIMESTAMP_PARAMETER = "nonce";
exports.BittrexApiClient = BittrexApiClient;
//# sourceMappingURL=BittrexApiClient.js.map